import {CachedPrimeMath, PrimeMath} from "./PrimeMath";


/**
 * Represents a particular way in which prime numbers can be arranged and drawn.
 */
export interface Model {
    get minX(): number | null;
    get maxX(): number | null;
    get minY(): number | null;
    get maxY(): number | null;


    /**
     * Returns `true` if and only if the square at coordinates (`x`, `y`) is a prime number.
     *
     * @param x the horizontal coordinate
     * @param y the vertical coordinate
     * @return `true` if and only if the square at coordinates (`x`, `y`) is a prime number
     */
    isPrime(x: number, y: number): boolean | null;
}

/**
 * A `Model` for a simple grid of primes.
 */
export class GridModel implements Model {
    /**
     * The instance to calculate primes with.
     *
     * @private
     */
    private readonly primeMath: PrimeMath = CachedPrimeMath.getInstance();

    get minX(): number | null {
        return 0;
    }
    get maxX(): number | null {
        return this.cols;
    }
    get minY(): number | null {
        return 0;
    }
    get maxY(): number | null {
        return null;
    }

    /**
     * The number of columns to draw primes in.
     */
    public cols: number = 100;


    isPrime(x: number, y: number): boolean | null {
        if (x < 0 || x >= this.cols || y < 0) return null;

        const n = y * this.cols + x + 1;
        return this.primeMath.isPrime(n);
    }
}
